@@ -22,11 +22,11 @@ import sys
 import tempfile
 
 from pystache import context
-import six
 import yaml
 
 from os_apply_config import collect_config
 from os_apply_config import config_exception as exc
+from os_apply_config import oac_file
 from os_apply_config import renderers
 from os_apply_config import value_types
 from os_apply_config import version
@@ -67,78 +67,6 @@ OS_CONFIG_FILES_PATH_OLD = '/var/run/os-collect-config/os_config_files.json'
 CONTROL_FILE_SUFFIX = ".oac"
 
 
-class OacFile(object):
-    DEFAULTS = {
-        'allow_empty': True,
-        'mode': None,
-    }
-
-    def __init__(self, body, **kwargs):
-        super(OacFile, self).__init__()
-        self.body = body
-
-        for k, v in six.iteritems(self.DEFAULTS):
-            setattr(self, '_' + k, v)
-
-        for k, v in six.iteritems(kwargs):
-            if not hasattr(self, k):
-                raise exc.ConfigException(
-                    "unrecognised file control key '%s'" % (k))
-            setattr(self, k, v)
-
-    def __eq__(self, other):
-        if type(other) is type(self):
-            return self.__dict__ == other.__dict__
-        return False
-
-    def __repr__(self):
-        a = ["OacFile(%s" % repr(self.body)]
-        for key, default in six.iteritems(self.DEFAULTS):
-            value = getattr(self, key)
-            if value != default:
-                a.append("%s=%s" % (key, repr(value)))
-        return ", ".join(a) + ")"
-
-    def set(self, key, value):
-        """Allows setting attrs as an expression rather than a statement."""
-        setattr(self, key, value)
-        return self
-
-    @property
-    def allow_empty(self):
-        """Returns allow_empty.
-
-        If True and body='', no file will be created and any existing
-        file will be deleted.
-        """
-        return self._allow_empty
-
-    @allow_empty.setter
-    def allow_empty(self, value):
-        if type(value) is not bool:
-            raise exc.ConfigException(
-                "allow_empty requires Boolean, got: '%s'" % value)
-        self._allow_empty = value
-        return self
-
-    @property
-    def mode(self):
-        """The permissions to set on the file, EG 0755."""
-        return self._mode
-
-    @mode.setter
-    def mode(self, v):
-        """Returns the file mode.
-
-        EG 0644. Must be between 0 and 0777, the sticky bit is not supported.
-        """
-        if type(v) is not int:
-            raise exc.ConfigException("mode '%s' is not numeric" % v)
-        if not 0 <= v <= 0o777:
-            raise exc.ConfigException("mode '%#o' out of range" % v)
-        self._mode = v
-
-
 def install_config(
         config_path, template_root, output_path, validate, subhash=None,
         fallback_metadata=None):
@@ -158,7 +86,15 @@ def print_key(
     for key in keys:
         try:
             config = config[key]
+            if config is None:
+                raise TypeError()
         except (KeyError, TypeError):
+            try:
+                if type(config) == list:
+                    config = config[int(key)]
+                    continue
+            except (IndexError, ValueError):
+                pass
             if default is not None:
                 print(str(default))
                 return
@@ -185,6 +121,10 @@ def write_file(path, obj):
     else:
         mode, uid, gid = 0o644, -1, -1
     mode = obj.mode or mode
+    if obj.owner is not None:
+        uid = obj.owner
+    if obj.group is not None:
+        gid = obj.group
 
     d = os.path.dirname(path)
     os.path.exists(d) or os.makedirs(d)
@@ -212,7 +152,7 @@ def build_tree(templates, config):
             if not isinstance(ctrl_dict, dict):
                 raise exc.ConfigException(
                     "header is not a dict: %s" % in_file)
-            res[out_file] = OacFile(body, **ctrl_dict)
+            res[out_file] = oac_file.OacFile(body, **ctrl_dict)
         except exc.ConfigException as e:
             e.args += in_file,
             raise
